/*
	This is sort of three demos in one.  I'm setting up a different example
	on each of three map levels.  Each will use an area/Toggle which can be
	clicked to toggle input.  I'm setting all of the area contents within the
	map files so you can ignore them.

	The values I am storing in each grid are simple booleans.  As the readMe
	file states, there's no reason to restrict yourself to that in a real
	project.
*/


////////////////////////////////////////////////////////////////////////////////
/*
	For the first example, I just want a visual indicator of changes to show
	the getting and setting of items.
*/
area/Toggle

	//	We need to declare a grid somewhere to use it.
	//	I'm setting the initial value of every cell to 0.
	var/grid2d_Grid/myGrid
	New(loc)
		..(loc)
		if (!src.myGrid)	src.myGrid	= new(world.maxx,world.maxy,0)

	//	Ignore how Toggle() is called.  It's not the point.
	proc/Toggle(X,Y)
		if ( !isnum(X) || !isnum(Y) )	return FALSE
		//	Get the item from the grid.
		var/item	= src.myGrid.grid2d_GetItem(X,Y)
		//	Set that cell to the toggled value.
		src.myGrid.grid2d_PlaceItem(!item,X,Y)
		//	These next two lines are for display and can be ignored for now.
		var/turf/T		= locate(X,Y,src.GetZ())
		T.icon_state	= "[!item]"
		return TRUE

////////////////////////////////////////////////////////////////////////////////
/*
	For the second example, I'm limiting selection to the lower left quadrant.
	The top left will then be that quadrant's grid flipped NORTH.  The top right
	will be the quadrant flipped NORTHEAST.  The bottom right will be the
	quadrant flipped EAST.
*/
area/Toggle/Flip

	New(loc)
		src.myGrid	= new(world.maxx/2,world.maxy/2,0)
		..(loc)

	Toggle(X,Y)
		//	Set the first quadrant's turf.
		.			= ..(X,Y)
		if ( !(.) )	return .
		var
			item	= src.myGrid.grid2d_GetItem(X,Y)
			turf/T
			flippedX
			flippedY
		//	Set the top left as a version flipped NORTH.
		flippedX	= src.myGrid.grid2d_GetFlipX(X,Y,NORTH)
		flippedY	= src.myGrid.grid2d_GetFlipY(X,Y,NORTH) + world.maxy/2
		T			= locate(flippedX,flippedY,src.GetZ())
		if (T)		T.icon_state	= "[item]"
		//	Set the top right as a version flipped NORTHEAST.
		//	If the length and width of the grid were not equal,
		//	this could result in garbage coordinates!
		flippedX	= src.myGrid.grid2d_GetFlipX(X,Y,NORTHEAST) + world.maxx/2
		flippedY	= src.myGrid.grid2d_GetFlipY(X,Y,NORTHEAST) + world.maxy/2
		T			= locate(flippedX,flippedY,src.GetZ())
		if (T)		T.icon_state	= "[item]"
		//	Set the bottom right as a version flipped EAST.
		flippedX	= src.myGrid.grid2d_GetFlipX(X,Y,EAST) + world.maxx/2
		flippedY	= src.myGrid.grid2d_GetFlipY(X,Y,EAST)
		if (T)		T	= locate(flippedX,flippedY,src.GetZ())
		//	...However, let's do something different.  We can make a new flipped
		//	version of the grid and reference that.  This is more slow, but it
		//	could be handy in a project where the results start to diverge.
		var/grid2d_Grid/flippedGrid	= src.myGrid.grid2d_GetFlipGrid(EAST)
		//	The flipped coordinates are the new grid's normal coordinates.
		//	However, we don't need the offet we added to find the turf.
		if (flippedGrid)
			flippedX	-= world.maxx/2
			item		= flippedGrid.grid2d_GetItem(flippedX,flippedY)
			if (T)		T.icon_state	= "[item]"

////////////////////////////////////////////////////////////////////////////////
/*
	For the third example, I'm splitting the map into 4 rectangles to display
	rotations when a grid's width and length differ.  The lower left is again
	the area of selection.  The top left is rotated 90 degrees, the top right
	is rotated 180 degrees and the bottom right is rotated 270 degrees.
*/
area/Toggle/Turn

	New(loc)
		src.myGrid	= new(world.maxx/2 - 5,world.maxy/2 + 5,0)
		..(loc)

	Toggle(X,Y)
		//	Set the lower left's turf.
		.			= ..(X,Y)
		if ( !(.) )	return .
		var
			item	= src.myGrid.grid2d_GetItem(X,Y)
			turf/T
			turnedX
			turnedY
		//	Set the top left as a version rotated 90 degrees.
		turnedX			= src.myGrid.grid2d_GetTurnX(X,Y,90)
		turnedY			= src.myGrid.grid2d_GetTurnY(X,Y,90) + world.maxy/2 + 5
		T				= locate(turnedX,turnedY,src.GetZ())
		T.icon_state	= "[item]"
		//	Set the top right as a version rotated 180 degrees.
		turnedX			= src.myGrid.grid2d_GetTurnX(X,Y,180) + world.maxy/2 + 5
		turnedY			= src.myGrid.grid2d_GetTurnY(X,Y,180) + world.maxx/2 - 5
		T				= locate(turnedX,turnedY,src.GetZ())
		T.icon_state	= "[item]"
		//	Set the bottom right as a version rotated 270 degrees.
		turnedX	= src.myGrid.grid2d_GetTurnX(X,Y,270) + world.maxx/2 - 5
		turnedY	= src.myGrid.grid2d_GetTurnY(X,Y,270)
		T		= locate(turnedX,turnedY,src.GetZ())
		//	...Again, let's do something different.  We can make a new rotated
		//	version of the grid and reference that.
		var/grid2d_Grid/turnedGrid	= src.myGrid.grid2d_GetTurnGrid(270)
		//	The turned coordinates are the new grid's normal coordinates.
		//	However, we don't need the offet we added to find the turf.
		if (turnedGrid)
			turnedX			-= world.maxx/2 - 5
			item			= turnedGrid.grid2d_GetItem(turnedX,turnedY)
			T.icon_state	= "[item]"

////////////////////////////////////////////////////////////////////////////////
/*
	Everything below this line has to do more with the interface than grids.
	You can ignore it.
*/


turf
	icon			= 'grid2d_turf.dmi'
	mouse_opacity	= 0
area
	icon			= 'grid2d_area.dmi'
	icon_state		= "0"
area/Toggle
	icon_state		= "1"
	mouse_opacity	= 2
	Click(location,control,params)
		if ( !isturf(location) )	return ..(location,control,params)
		var/turf/T	= location
		src.Toggle(T.x,T.y)
		return ..(location,control,params)
	proc/GetZ()
		var/atom/A	= locate(src.type) in world.contents
		A			= A.contents[1]
		return A.z
area/Toggle/Set

client
	control_freak	= TRUE
	perspective	= EDGE_PERSPECTIVE|EYE_PERSPECTIVE
	view		= "70x70"
	New()
		..()
		src.eye	= locate(world.maxx/2,world.maxy/2,1)
	verb
		Set()
			var/area/Toggle/A	= locate(/area/Toggle/Set) in world.contents
			src.eye				= locate(world.maxx/2,world.maxy/2,A.GetZ())
		Flip()
			var/area/Toggle/A	= locate(/area/Toggle/Flip) in world.contents
			src.eye				= locate(world.maxx/2,world.maxy/2,A.GetZ())
		Turn()
			var/area/Toggle/A	= locate(/area/Toggle/Turn) in world.contents
			src.eye				= locate(world.maxx/2,world.maxy/2,A.GetZ())
