grid2d_Grid

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns the direction from the coordinates FromX,FromY to ToX,ToY.
		//	By default, this is implemented to return NORTH, EAST, SOUTH and WEST
		//	only when a coordinate is shared (just like get_dir()).
		grid2d_GetDir(FromX,FromY,ToX,ToY)
			if ( !isnum(FromX) || !isnum(FromY) || !isnum(ToX) || !isnum(ToY) )
				return 0
			FromY	= round(FromY)
			ToY		= round(ToY)
			var/val
			//	Wrap back around vertical edge?
			if ( src.grid2d_DoesWrapX() && (abs(ToX-FromX) > (src.grid2d_GetMaxX()/2)) )
				val		= src.grid2d_GetWrapX(FromX)
				FromX	= src.grid2d_GetWrapX(ToX)
				ToX		= val
			else
				FromX	= round(FromX)
				ToX		= round(ToX)
				val								= src.grid2d_GetMaxX()
				if ( (FromX>val) || (ToX>val) )	return 0
			//	Wrap back around horizontal edge?
			if ( src.grid2d_DoesWrapY() && (abs(ToY-FromY) > (src.grid2d_GetMaxY()/2)) )
				val		= src.grid2d_GetWrapY(FromY)
				FromY	= src.grid2d_GetWrapY(ToY)
				ToY		= val
				val								= src.grid2d_GetMaxY()
				if ( (FromY>val) || (ToY>val) )	return 0
			//	Even wrapping can provide low output if given bad input.
			if ( (FromX<1) || (FromY<1) || (ToX<1) || (ToY<1) )	return 0
			//	Follow BYOND conventions.
			if (ToX>FromX)
				if (ToY>FromY)	return NORTHEAST
				if (ToY<FromY)	return SOUTHEAST
				return EAST
			if (ToX<FromX)
				if (ToY>FromY)	return NORTHWEST
				if (ToY<FromY)	return SOUTHWEST
				return WEST
			if (ToY>FromY)	return NORTH
			if (ToY<FromY)	return SOUTH
			return 0

		//----------------------------------------------------------------------
		//	Returns the horizontal coordinate of the cell NumSteps in the
		//	direction of Dir from the cell X,Y.
		grid2d_GetXInDir(X,Y,Dir,NumSteps=1)
			if (!isnum(Dir) || !isnum(X) || !isnum(Y) || !isnum(NumSteps))		return 0
			NumSteps															= round(NumSteps)
			if ( (Dir==EAST) || (Dir==NORTHEAST) || (Dir==SOUTHEAST) )			X	+= NumSteps
			else if ( (Dir==WEST) || (Dir==NORTHWEST) || (Dir==SOUTHWEST) )		X	-= NumSteps
			if ( src.grid2d_DoesWrapX() )										X	= src.grid2d_GetWrapX(X)
			return round(X)

		//----------------------------------------------------------------------
		//	Returns the vertical coordinate of the cell NumSteps in the
		//	direction of Dir from the cell X,Y.
		grid2d_GetYInDir(X,Y,Dir,NumSteps=1)
			if (!isnum(Dir) || !isnum(X) || !isnum(Y) || !isnum(NumSteps))		return 0
			NumSteps															= round(NumSteps)
			if ( (Dir==NORTH) || (Dir==NORTHEAST) || (Dir==NORTHWEST) )			Y	+= NumSteps
			else if ( (Dir==SOUTH) || (Dir==SOUTHEAST) || (Dir==SOUTHWEST) )	Y	-= NumSteps
			if ( src.grid2d_DoesWrapY() )										Y	= src.grid2d_GetWrapY(Y)
			return round(Y)
