grid2d_Grid

	//==========================================================================
	var
		const
			grid2d_NONE			= 0
			grid2d_NORTH 		= 1
			grid2d_NORTHEAST	= 2
			grid2d_EAST 		= 4
			grid2d_SOUTHEAST	= 8
			grid2d_SOUTH 		= 16
			grid2d_SOUTHWEST	= 32
			grid2d_WEST 		= 64
			grid2d_NORTHWEST	= 128

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns a list of the directional bits set in the num Bits.
		grid2d_BitsToDirBitList(Bits)
			if ( !isnum(Bits) )				return list()
			.								= list()
			if (Bits&src.grid2d_NORTH)		.	+= src.grid2d_NORTH
			if (Bits&src.grid2d_NORTHEAST)	.	+= src.grid2d_NORTHEAST
			if (Bits&src.grid2d_EAST)		.	+= src.grid2d_EAST
			if (Bits&src.grid2d_SOUTHEAST)	.	+= src.grid2d_SOUTHEAST
			if (Bits&src.grid2d_SOUTH)		.	+= src.grid2d_SOUTH
			if (Bits&src.grid2d_SOUTHWEST)	.	+= src.grid2d_SOUTHWEST
			if (Bits&src.grid2d_WEST)		.	+= src.grid2d_WEST
			if (Bits&src.grid2d_NORTHWEST)	.	+= src.grid2d_NORTHWEST

		//----------------------------------------------------------------------
		//	Returns a list of the directions defined by the num Bits.
		grid2d_BitsToDirList(Bits)
			if ( !isnum(Bits) )				return list()
			.								= list()
			if (Bits&src.grid2d_NORTH)		.	+= NORTH
			if (Bits&src.grid2d_NORTHEAST)	.	+= NORTHEAST
			if (Bits&src.grid2d_EAST)		.	+= EAST
			if (Bits&src.grid2d_SOUTHEAST)	.	+= SOUTHEAST
			if (Bits&src.grid2d_SOUTH)		.	+= SOUTH
			if (Bits&src.grid2d_SOUTHWEST)	.	+= SOUTHWEST
			if (Bits&src.grid2d_WEST)		.	+= WEST
			if (Bits&src.grid2d_NORTHWEST)	.	+= NORTHWEST

		//----------------------------------------------------------------------
		//	Each cell stores a collection of bits representing its connections.
		//	This returns the BYOND direction corresponding with a single DirBit.
		//	Multiple DirBits and other invalid input will return 0.
		grid2d_DirBitToDir(DirBit)
			if ( !isnum(DirBit) )				return 0
			if (DirBit==src.grid2d_NORTH)		return NORTH
			if (DirBit==src.grid2d_NORTHEAST)	return NORTHEAST
			if (DirBit==src.grid2d_EAST)		return EAST
			if (DirBit==src.grid2d_SOUTHEAST)	return SOUTHEAST
			if (DirBit==src.grid2d_SOUTH)		return SOUTH
			if (DirBit==src.grid2d_SOUTHWEST)	return SOUTHWEST
			if (DirBit==src.grid2d_WEST)		return WEST
			if (DirBit==src.grid2d_NORTHWEST)	return NORTHWEST
			return 0

		//----------------------------------------------------------------------
		//	Returns the name of the direction bit.
		grid2d_DirBitToText(Bit)
			if ( !isnum(Bit) )				return ""
			if (Bit==src.grid2d_NORTH)		return "NORTH"
			if (Bit==src.grid2d_NORTHEAST)	return "NORTHEAST"
			if (Bit==src.grid2d_EAST)		return "EAST"
			if (Bit==src.grid2d_SOUTHEAST)	return "SOUTHEAST"
			if (Bit==src.grid2d_SOUTH)		return "SOUTH"
			if (Bit==src.grid2d_SOUTHWEST)	return "SOUTHWEST"
			if (Bit==src.grid2d_WEST)		return "WEST"
			if (Bit==src.grid2d_NORTHWEST)	return "NORTHWEST"
			return ""

		//----------------------------------------------------------------------
		//	Returns the name of the direction.
		grid2d_DirToText(Dir)
			if ( !isnum(Dir) )	return ""
			if (Dir==NORTH)		return "NORTH"
			if (Dir==NORTHEAST)	return "NORTHEAST"
			if (Dir==EAST)		return "EAST"
			if (Dir==SOUTHEAST)	return "SOUTHEAST"
			if (Dir==SOUTH)		return "SOUTH"
			if (Dir==SOUTHWEST)	return "SOUTHWEST"
			if (Dir==WEST)		return "WEST"
			if (Dir==NORTHWEST)	return "NORTHWEST"
			return ""

		//----------------------------------------------------------------------
		//	Returns the bit corresponding to direction Dir.
		grid2d_DirToDirBit(Dir)
			if ( !isnum(Dir) )	return src.grid2d_NONE
			if (Dir==NORTH)		return src.grid2d_NORTH
			if (Dir==NORTHEAST)	return src.grid2d_NORTHEAST
			if (Dir==EAST)		return src.grid2d_EAST
			if (Dir==SOUTHEAST)	return src.grid2d_SOUTHEAST
			if (Dir==SOUTH)		return src.grid2d_SOUTH
			if (Dir==SOUTHWEST)	return src.grid2d_SOUTHWEST
			if (Dir==WEST)		return src.grid2d_WEST
			if (Dir==NORTHWEST)	return src.grid2d_NORTHWEST
			return src.grid2d_NONE

		//----------------------------------------------------------------------
		//	Returns the quantity of directional bits in the num Bits.
		grid2d_GetNumDirBits(Bits)
			if ( !isnum(Bits) )				return 0
			.								= 0
			if (Bits&src.grid2d_NORTH)		.	+= 1
			if (Bits&src.grid2d_NORTHEAST)	.	+= 1
			if (Bits&src.grid2d_EAST)		.	+= 1
			if (Bits&src.grid2d_SOUTHEAST)	.	+= 1
			if (Bits&src.grid2d_SOUTH)		.	+= 1
			if (Bits&src.grid2d_SOUTHWEST)	.	+= 1
			if (Bits&src.grid2d_WEST)		.	+= 1
			if (Bits&src.grid2d_NORTHWEST)	.	+= 1
