grid2d_Grid

	//==========================================================================
	proc
		/*
			Flipping performs the following transformations depending on the
			direction of the flip.

				origin		N/S		E/W		Ne/Sw	Se/Nw
				ABC			GHI		CBA		ADG		IFC
				DEF			DEF		FED		BEH		HEB
				GHI			ABC		IHG		CFI		GDA

			Diagonally flipping a grid with an unequal width and length will fail.
		*/

		//----------------------------------------------------------------------
		//	Find the horizontal coordinate of a cell with coordinate X as if the
		//	grid was flipped in direction Dir.  Returns 0 on failure.
		grid2d_GetFlipX(X,Y,Dir)
			if ( !src.grid2d_HasCoordinates(X,Y) )			return 0
			if ( !isnum(Dir) )								return 0
			if ( src.grid2d_DoesWrapX() )					X	= src.grid2d_GetWrapX(X)
			if (!Dir)										return X
			if ( (Dir==NORTH) || (Dir==SOUTH) )				return X
			if ( (Dir==EAST) || (Dir==WEST) )				return src.grid2d_GetMaxX() - X + 1
			//	Need equal lengths for decent diagonal flipping.
			if (src.grid2d_GetMaxX() != src.grid2d_GetMaxY())	return 0
			//	Find the X on the line to divide along. (slope*Y + origin)
			//	Make sure the slope will be defined.
			var/slope	= src.grid2d_GetMaxX() - 1
			if (!slope)										return 0
			if ( src.grid2d_DoesWrapY() )					Y	= src.grid2d_GetWrapY(Y)
			//	Negative slope and origin is Xmax,1?
			if ( (Dir==NORTHEAST) || (Dir==SOUTHWEST) )
				slope	= (1 - src.grid2d_GetMaxY())/slope
				if ( X==(Y*slope + src.grid2d_GetMaxX()) )	return X
				return src.grid2d_GetMaxY() - Y + 1
			//	Positive slope and origin is 1,1?
			if ( (Dir==SOUTHEAST) || (Dir==NORTHWEST) )
				slope	= (src.grid2d_GetMaxY() - 1)/slope
				if ( X==(Y*slope + 1) )						return X
				return src.grid2d_GetMaxY() - Y + 1
			//	Bad direction.
			return 0

		//----------------------------------------------------------------------
		//	Find the vertical coordinate of a cell with coordinate Y as if the
		//	grid was flipped in direction Dir.  Returns 0 on failure.
		grid2d_GetFlipY(X,Y,Dir)
			if ( !src.grid2d_HasCoordinates(1,Y) )			return 0
			if ( !isnum(Dir) )								return 0
			if ( src.grid2d_DoesWrapY() )					Y	= src.grid2d_GetWrapY(Y)
			if (!Dir)										return Y
			if ( (Dir==EAST) || (Dir==WEST) )				return Y
			if ( (Dir==NORTH) || (Dir==SOUTH) )				return src.grid2d_GetMaxY() - Y + 1
			//	Need equal lengths for decent diagonal flipping.
			if (src.grid2d_GetMaxX() != src.grid2d_GetMaxY())	return 0
			//	X		= Yline*slope +Xorigin.
			//	Yline	= (X - Xorigin)/slope
			var/slope	= src.grid2d_GetMaxX() - 1
			if (!slope)										return 0
			if ( src.grid2d_DoesWrapX() )					X	= src.grid2d_GetWrapX(X)
			//	Negative slope and origin is Xmax,1?
			if ( (Dir==NORTHEAST) || (Dir==SOUTHWEST) )
				slope	= (1 - src.grid2d_GetMaxY())/slope
				if (!slope)									return 0
				if (Y == ((X-src.grid2d_GetMaxX())/slope) )	return Y
				return src.grid2d_GetMaxX() - X + 1
			//	Positive slope and origin is 1,1?
			if ( (Dir==SOUTHEAST) || (Dir==NORTHWEST) )
				slope	= (src.grid2d_GetMaxY() - 1)/slope
				if (!slope)									return 0
				if (Y == ((X-1)/slope) )					return Y
				return src.grid2d_GetMaxX() - X + 1
			//	Bad direction.
			return 0

		//----------------------------------------------------------------------
		//	Returns a grid flipped in direction Dir.  Returns null on failure.
		grid2d_GetFlipGrid(Dir)
			var
				grid2d_Grid/result
				X
				Y
				maxX	= src.grid2d_GetMaxX()
				maxY	= src.grid2d_GetMaxY()
			//	Along a horizontal axis.
			if ( (Dir==NORTH) || (Dir==SOUTH) )
				result	= new src.type (maxX,maxY)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),X,maxY-Y+1)
				return result
			//	Along a vertical axis.
			if ( (Dir==EAST) || (Dir==WEST) )
				result	= new src.type (maxX,maxY)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),maxX-X+1,Y)
				return result
			//	Need equal lengths for decent diagonal flipping.
			if (maxX != maxY)	return null
			//	Find the X on the line to divide along.
			//	Make sure the slope will be defined.
			//	X		= Yline*slope +Xorigin.
			//	Yline	= (X - Xorigin)/slope
			var/slope	= src.grid2d_GetMaxX() - 1
			if (!slope)	return null
			var
				lineX
				lineY
			//	Negative slope and origin is Xmax,1?
			if ( (Dir==NORTHEAST) || (Dir==SOUTHWEST) )
				slope		= (1 - maxY)/slope
				if (!slope)	return src.grid2d_Copy()
				result		= new src.type (maxX,maxY)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						lineX	= Y*slope + maxX
						lineY	= (X - maxX)/slope
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),(X==lineX)?(X):(maxY-Y+1),(Y==lineY)?(Y):(maxX-X+1))
				return result
			//	Positive slope and origin is 1,1?
			if ( (Dir==SOUTHEAST) || (Dir==NORTHWEST) )
				slope		= (maxY - 1)/slope
				if (!slope)	return src.grid2d_Copy()
				result		= new src.type (maxX,maxY)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						lineX	= Y*slope + maxX
						lineY	= (X - maxX)/slope
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),(X==lineX)?(X):(maxY-Y+1),(Y==lineY)?(Y):(maxX-X+1))
				return result
			return null
