grid2d_Grid

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns a list of indices representing the cells surrounding cell Index.
		grid2d_GetNeighborIndices(Index)
			if ( !src.grid2d_HasIndex(Index) )	return list()
			//	Find all contained coordinates within a distance of 1.
			var
				X	= src.grid2d_IndexToX(Index)
				Y	= src.grid2d_IndexToY(Index)
			.		= list()
			if ( (src.grid2d_nDirs&src.grid2d_NORTH) && src.grid2d_HasCoordinates(X,Y+1) )
				.	+= src.grid2d_XYToIndex(X,Y+1)
			if ( (src.grid2d_nDirs&src.grid2d_NORTHEAST) && src.grid2d_HasCoordinates(X+1,Y+1) )
				.	+= src.grid2d_XYToIndex(X+1,Y+1)
			if ( (src.grid2d_nDirs&src.grid2d_EAST) && src.grid2d_HasCoordinates(X+1,Y) )
				.	+= src.grid2d_XYToIndex(X+1,Y)
			if ( (src.grid2d_nDirs&src.grid2d_SOUTHEAST) && src.grid2d_HasCoordinates(X+1,Y-1) )
				.	+= src.grid2d_XYToIndex(X+1,Y-1)
			if ( (src.grid2d_nDirs&src.grid2d_SOUTH) && src.grid2d_HasCoordinates(X,Y-1) )
				.	+= src.grid2d_XYToIndex(X,Y-1)
			if ( (src.grid2d_nDirs&src.grid2d_SOUTHWEST) && src.grid2d_HasCoordinates(X-1,Y-1) )
				.	+= src.grid2d_XYToIndex(X-1,Y-1)
			if ( (src.grid2d_nDirs&src.grid2d_WEST) && src.grid2d_HasCoordinates(X-1,Y) )
				.	+= src.grid2d_XYToIndex(X-1,Y)
			if ( (src.grid2d_nDirs&src.grid2d_NORTHWEST) && src.grid2d_HasCoordinates(X-1,Y+1) )
				.	+= src.grid2d_XYToIndex(X-1,Y+1)

		//----------------------------------------------------------------------
		//	Returns a list of valid storage indices.
		grid2d_GetIndices()
			.											= list()
			for(var/index = 1 to src.grid2d_cells.len)	.	+= index

		//----------------------------------------------------------------------
		//	Returns whether or not Index is within bounds.
		grid2d_HasIndex(Index)
			//	Must be an integer.
			if ( !isnum(Index) || (Index!=round(Index)) )	return FALSE
			//	Must be in bounds.
			return ( (Index>0) && (Index<=src.grid2d_cells.len) )

		//----------------------------------------------------------------------
		//	Returns the X coordinate of the cell at Index.
		//	NOTE: Assumes internals of GetSize procs!
		grid2d_IndexToX(Index)
			if ( !isnum(Index) || (Index<1) || (Index>src.grid2d_cells.len) )
				return 0
			var/Y				= Index/src.grid2d_X
			if (Y != round(Y))	Y	= round(Y) + 1
			return Index - (Y-1)*src.grid2d_X

		//----------------------------------------------------------------------
		//	Returns the Y coordinate of the cell at Index.
		//	NOTE: Assumes internals of GetSize procs!
		grid2d_IndexToY(Index)
			if ( !isnum(Index) || (Index<1) || (Index>src.grid2d_cells.len) )
				return 0
			.					= Index/src.grid2d_X
			if (. != round(.))	.	= round(.) + 1

		//----------------------------------------------------------------------
		//	This returns the cell index at the coordinates X and Y.
		//	Returns 0 if coordinates are out of the grid's bounds.
		//	NOTE: Assumes internals of GetSize and DoesWrap procs!
		grid2d_XYToIndex(X,Y)
			if ( !isnum(X) || !isnum(Y) )			return 0
			if (src.grid2d_wrap&src.grid2d_WRAP_X)	X	= src.grid2d_GetWrapX(X)
			else if ( (X<1) || (X>src.grid2d_X) )	return 0
			if (src.grid2d_wrap&src.grid2d_WRAP_Y)	Y	= src.grid2d_GetWrapY(Y)
			else if ( (Y<1) || (Y>src.grid2d_cells.len/src.grid2d_X) )	return 0
			return max(0,(Y-1)*src.grid2d_X + X)
