grid2d_Grid

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns a list of the coordinates of every position Item is stored.
		//	list(X1,Y1,X2,Y2,X3,Y3...)
		grid2d_GetCoordinates(Item)
			.			= list()
			var/index	= src.grid2d_cells.Find(Item)
			while(index)
				.		+= src.grid2d_IndexToX(index)
				.		+= src.grid2d_IndexToY(index)
				index	= src.grid2d_cells.Find(Item,index+1,0)

		//----------------------------------------------------------------------
		//	Returns whatever is stored in the grid at the coordinates X,Y.
		grid2d_GetItem(X,Y)
			var/index	= src.grid2d_XYToIndex(X,Y)
			if (index)	return src.grid2d_cells[index]
			return null

		//----------------------------------------------------------------------
		//	Returns a list containing each item stored in the grid.  If JustOnce,
		//	multiple appearances of the same item will be filtered out of the result.
		grid2d_GetItems(JustOnce=FALSE)
			if (!src.grid2d_cells)	return list()
			if (JustOnce)
				.			= list()
				for(var/index = 1 to src.grid2d_cells.len)
					if ( !(src.grid2d_cells[index] in .) )
						.	+= src.grid2d_cells[index]
				return .
			return src.grid2d_cells.Copy()

		//----------------------------------------------------------------------
		//	Returns the number of times Item is stored in the grid.
		grid2d_GetNumOccurances(Item)
			.			= 0
			var/index	= src.grid2d_cells.Find(Item)
			while(index)
				.						+= 1
				index	= src.grid2d_cells.Find(Item,index+1,0)

		//----------------------------------------------------------------------
		//	Returns the X coordinate of the Nth occurance of item.  N=1 is the
		//	first item and N=0 is the last.  Negative values count away from the
		//	last item.  The occurances are in the order of those returned by
		//	grid2d_GetCoordinates().
		grid2d_GetNthX(Item,N=1)
			if ( !isnum(N) )	return 0
			N					= round(N)
			if (N<=0)			N	+= src.grid2d_GetNumOccurances(Item)
			if (N<=0)			return 0
			var/index	= src.grid2d_cells.Find(Item)
			N--
			while(index && (N>0))
				N--
				index	= src.grid2d_cells.Find(Item,index+1,0)
			return src.grid2d_IndexToX(index)

		//----------------------------------------------------------------------
		//	Returns the Y coordinate of the Nth occurance of item.  N=1 is the
		//	first item and N=0 is the last.  Negative values count away from the
		//	last item.  The occurances are in the order of those returned by
		//	grid2d_GetCoordinates().
		grid2d_GetNthY(Item,N=1)
			if ( !isnum(N) )	return 0
			N					= round(N)
			if (N<=0)			N	+= src.grid2d_GetNumOccurances(Item)
			if (N<=0)			return 0
			var/index	= src.grid2d_cells.Find(Item)
			N--
			while(index && (N>0))
				N--
				index	= src.grid2d_cells.Find(Item,index+1,0)
			return src.grid2d_IndexToY(index)

		//----------------------------------------------------------------------
		//	Returns whether or not Item is anywhere in the grid.
		grid2d_HasItem(Item)
			return (src.grid2d_cells.Find(Item)>0)

		//----------------------------------------------------------------------
		//	Stores Item at the coordinates X,Y and returns TRUE on success.
		//	Null is as valid an item as any other.
		grid2d_PlaceItem(Item,X,Y)
			var/index	= src.grid2d_XYToIndex(X,Y)
			if (index)
				src.grid2d_cells[index]	= Item
				return TRUE
			return FALSE

		//----------------------------------------------------------------------
		//	This puts Replacement wherever OldItem is stored and returns the
		//	number of times it happens
		grid2d_Replace(OldItem,Replacement)
			.			= 0
			var/index	= src.grid2d_cells.Find(OldItem)
			while(index)
				src.grid2d_cells[index]	= Replacement
				.						+= 1
				index	= src.grid2d_cells.Find(OldItem,index+1,0)

		//----------------------------------------------------------------------
		//	Swaps the positions of the objects located at X1,Y1 and X2,Y2.
		//	Returns TRUE on success.
		grid2d_Swap(X1,Y1,X2,Y2)
			var/index1		= src.grid2d_XYToIndex(X1,Y1)
			if (index1<1)	return FALSE
			var/index2		= src.grid2d_XYToIndex(X2,Y2)
			if (index2<1)	return FALSE
			src.grid2d_cells.Swap(index1,index2)
			return TRUE
