grid2d_Grid

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns a list containing the items listed in the same coordinates
		//	of the src grid and	Grid.  If JustOnce, multiple appearances of the
		//	same item will be filtered out of the result.
		grid2d_GetItemsMatchedByXY(grid2d_Grid/Grid,JustOnce=FALSE)
			if ( !istype(Grid,/grid2d_Grid) )	return list()
			if (Grid==src)	return src.grid2d_GetItems(JustOnce)
			if ( !length(src.grid2d_cells) || !length(Grid.grid2d_cells) )
				return list()
			var
				maxX	= min(src.grid2d_X,Grid.grid2d_X)
				maxY	= min(src.grid2d_cells.len/src.grid2d_X,Grid.grid2d_cells.len/Grid.grid2d_X)
				X
				Y
				index
			.			= list()
			if (JustOnce)
				for(Y = 1 to maxY)
					for(X = 1 to maxX)
						index	= src.grid2d_XYToIndex(X,Y)
						if (src.grid2d_cells[index] == Grid.grid2d_cells[Grid.grid2d_XYToIndex(X,Y)])
							if ( !(src.grid2d_cells[index] in .) )
								.	+= src.grid2d_cells[index]
			else
				for(Y = 1 to maxY)
					for(X = 1 to maxX)
						index	= src.grid2d_XYToIndex(X,Y)
						if (src.grid2d_cells[index] == Grid.grid2d_cells[Grid.grid2d_XYToIndex(X,Y)])
							.	+= src.grid2d_cells[index]


		//----------------------------------------------------------------------
		//	Returns a list of coordinates which have matching items in both the
		//	src grid and Grid.  If JustOnce, only the first X and Y of each
		//	item found will be in the result.
		grid2d_GetMatchingItemXYs(grid2d_Grid/Grid,JustOnce=FALSE)
			if ( !istype(Grid,/grid2d_Grid) )	return list()
			if ( !length(src.grid2d_cells) || !length(Grid.grid2d_cells) )
				return list()
			var
				maxX	= min(src.grid2d_X,Grid.grid2d_X)
				maxY	= min(src.grid2d_cells.len/src.grid2d_X,Grid.grid2d_cells.len/Grid.grid2d_X)
				X
				Y
				index
			.			= list()
			if (JustOnce)
				var/list/items	= list()
				for(Y = 1 to maxY)
					for(X = 1 to maxX)
						index	= src.grid2d_XYToIndex(X,Y)
						if (src.grid2d_cells[index] == Grid.grid2d_cells[Grid.grid2d_XYToIndex(X,Y)])
							if ( !(src.grid2d_cells[index] in items) )
								items	+= src.grid2d_cells[index]
								.		+= X
								.		+= Y
			else
				for(Y = 1 to maxY)
					for(X = 1 to maxX)
						index	= src.grid2d_XYToIndex(X,Y)
						if (src.grid2d_cells[index] == Grid.grid2d_cells[Grid.grid2d_XYToIndex(X,Y)])
							.	+= X
							.	+= Y

		//----------------------------------------------------------------------
		//	Compares the src grid with Grid.  Returns true if the items in every
		//	cell of the smaller grid2d_Grid are in the same positions in both grids.
		grid2d_XYItemsAllMatch(grid2d_Grid/Grid)
			if ( !istype(Grid,/grid2d_Grid) )	return FALSE
			if (Grid==src)						return TRUE
			//	Same dimensions?
			if ( (Grid.grid2d_X==src.grid2d_X) && (Grid.grid2d_cells.len==src.grid2d_cells.len) )
				for(var/index = 1 to src.grid2d_cells.len)
					if (Grid.grid2d_cells[index] != src.grid2d_cells[index])
						return FALSE
				return TRUE
			//	Different dimensions.
			var
				maxX	= min(src.grid2d_X,Grid.grid2d_X)
				maxY	= min(src.grid2d_cells.len/src.grid2d_X,Grid.grid2d_cells.len/Grid.grid2d_X)
				X
				Y
			for(Y = 1 to maxY)
				for(X = 1 to maxX)
					if (src.grid2d_cells[src.grid2d_XYToIndex(X,Y)]!=Grid.grid2d_cells[Grid.grid2d_XYToIndex(X,Y)])
						return FALSE
			return TRUE

