grid2d_Grid

	//==========================================================================
	var
		grid2d_nDirs	= grid2d_NORTH|grid2d_NORTHEAST|grid2d_EAST|grid2d_SOUTHEAST|grid2d_SOUTH|grid2d_SOUTHWEST|grid2d_WEST|grid2d_NORTHWEST

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns TRUE is the cells at X1,Y1 and X2,Y2 are right next to each other.
		grid2d_AreNeighbors(X1,Y1,X2,Y2)
			//	Must be actual grid coordinates.
			if ( !src.grid2d_HasCoordinates(X1,Y1) || !src.grid2d_HasCoordinates(X2,Y2) )
				return FALSE
			//	Must be right next to each other.
			if (src.grid2d_GetDist(X1,Y1,X2,Y2) != 1)	return FALSE
			var/D				= src.grid2d_GetDir(X1,Y1,X2,Y2)
			if (D==NORTH)		return src.grid2d_nDirs&src.grid2d_NORTH
			if (D==NORTHEAST)	return src.grid2d_nDirs&src.grid2d_NORTHEAST
			if (D==EAST)		return src.grid2d_nDirs&src.grid2d_EAST
			if (D==SOUTHEAST)	return src.grid2d_nDirs&src.grid2d_SOUTHEAST
			if (D==SOUTH)		return src.grid2d_nDirs&src.grid2d_SOUTH
			if (D==SOUTHWEST)	return src.grid2d_nDirs&src.grid2d_SOUTHWEST
			if (D==WEST)		return src.grid2d_nDirs&src.grid2d_WEST
			if (D==NORTHWEST)	return src.grid2d_nDirs&src.grid2d_NORTHWEST
			return FALSE


		//----------------------------------------------------------------------
		//	Returns a list of coordinates representing the cells surrounding X,Y.
		grid2d_GetNeighbors(X,Y)
			if ( !src.grid2d_HasCoordinates(X,Y) )	return list()
			//	Find all contained coordinates within a distance of 1.
			.		= list()
			if ( (src.grid2d_nDirs&src.grid2d_NORTH) && src.grid2d_HasCoordinates(X,Y+1) )
				.	+= X
				.	+= Y+1
			if ( (src.grid2d_nDirs&src.grid2d_NORTHEAST) && src.grid2d_HasCoordinates(X+1,Y+1) )
				.	+= X+1
				.	+= Y+1
			if ( (src.grid2d_nDirs&src.grid2d_EAST) && src.grid2d_HasCoordinates(X+1,Y) )
				.	+= X+1
				.	+= Y
			if ( (src.grid2d_nDirs&src.grid2d_SOUTHEAST) && src.grid2d_HasCoordinates(X+1,Y-1) )
				.	+= X+1
				.	+= Y-1
			if ( (src.grid2d_nDirs&src.grid2d_SOUTH) && src.grid2d_HasCoordinates(X,Y-1) )
				.	+= X
				.	+= Y-1
			if ( (src.grid2d_nDirs&src.grid2d_SOUTHWEST) && src.grid2d_HasCoordinates(X-1,Y-1) )
				.	+= X-1
				.	+= Y-1
			if ( (src.grid2d_nDirs&src.grid2d_WEST) && src.grid2d_HasCoordinates(X-1,Y) )
				.	+= X-1
				.	+= Y
			if ( (src.grid2d_nDirs&src.grid2d_NORTHWEST) && src.grid2d_HasCoordinates(X-1,Y+1) )
				.	+= X-1
				.	+= Y+1
			//	Wrap.
			var
				list/results
				index
			if ( src.grid2d_DoesWrapX() )
				results	= .
				index	= length(results)/2
				while(index>0)
					results[2*index-1]	= src.grid2d_GetWrapX(results[2*index-1])
					index--
			if ( src.grid2d_DoesWrapY() )
				results	= .
				index	= length(results)/2
				while(index>0)
					results[2*index]	= src.grid2d_GetWrapY(results[2*index])
					index--


		//----------------------------------------------------------------------
		//	Returns the list of directions cells are acknowledged to have
		//	neighbors in.
		grid2d_GetNeighborDirs()
			return src.grid2d_BitsToDirList(src.grid2d_nDirs)

		//----------------------------------------------------------------------
		//	Returns whether or not cells acknowledge neighbors in direction Dir.
		grid2d_IsNeighborDir(Dir)
			if ( !isnum(Dir) )	return FALSE
			if (Dir==NORTH)		return src.grid2d_nDirs&src.grid2d_NORTH
			if (Dir==NORTHEAST)	return src.grid2d_nDirs&src.grid2d_NORTHEAST
			if (Dir==EAST)		return src.grid2d_nDirs&src.grid2d_EAST
			if (Dir==SOUTHEAST)	return src.grid2d_nDirs&src.grid2d_SOUTHEAST
			if (Dir==SOUTH)		return src.grid2d_nDirs&src.grid2d_SOUTH
			if (Dir==SOUTHWEST)	return src.grid2d_nDirs&src.grid2d_SOUTHWEST
			if (Dir==WEST)		return src.grid2d_nDirs&src.grid2d_WEST
			if (Dir==NORTHWEST)	return src.grid2d_nDirs&src.grid2d_NORTHWEST
			return FALSE

		//----------------------------------------------------------------------
		//	This accepts either a single argument which is a list of directions
		//	or any number of arguments which are directions themselves.  This
		//	proc forces the grid to acknowledge neighbors in and only in the
		//	given directions.
		grid2d_SetNeighborDirs()
			if (!args.len)					return FALSE
			var/list/dirs
			if ( istype(args[1],/list) )	dirs	= args[1]
			else							dirs	= args
			src.grid2d_nDirs			= src.grid2d_NONE
			if ( dirs.Find(NORTH) )		src.grid2d_nDirs	|= src.grid2d_NORTH
			if ( dirs.Find(NORTHEAST) )	src.grid2d_nDirs	|= src.grid2d_NORTHEAST
			if ( dirs.Find(EAST) )		src.grid2d_nDirs	|= src.grid2d_EAST
			if ( dirs.Find(SOUTHEAST) )	src.grid2d_nDirs	|= src.grid2d_SOUTHEAST
			if ( dirs.Find(SOUTH) )		src.grid2d_nDirs	|= src.grid2d_SOUTH
			if ( dirs.Find(SOUTHWEST) )	src.grid2d_nDirs	|= src.grid2d_SOUTHWEST
			if ( dirs.Find(WEST) )		src.grid2d_nDirs	|= src.grid2d_WEST
			if ( dirs.Find(NORTHWEST) )	src.grid2d_nDirs	|= src.grid2d_NORTHWEST
			return TRUE


