grid2d_Grid

/*
	Shifting moves multiple items in the grid.  Wrapping is assumed.
*/

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Moves the item at OriginX,OriginY the specified horizontal and vertical
		//	amounts over.  Items in cells along the line formed will be similarly moved.
		//	However, it will skip the spaces between offsets.
		//
		//		grid2d_Shift(2,2,1,0)		grid2d_Shift(2,2,0,1)
		//
		//			ABCD		ABCD		ABCD		AFCD
		//			EFGH	=>	EFGH		EFGH	=>	EJGH
		//			IJKL		LIJK		IJKL		INKL
		//			MNOP		MNOP		MNOP		MBOP
		//
		//		grid2d_Shift(2,2,2,0)		grid2d_Shift(2,2,0,2)
		//
		//			ABCD		ABCD		ABCD		AJCD
		//			EFGH	=>	EFGH		EFGH	=>	EFGH
		//			IJKL		ILKJ		IJKL		IBKL
		//			MNOP		MNOP		MNOP		MNOP
		//
		//		grid2d_Shift(2,2,1,1)		grid2d_Shift(2,2,1,2)
		//
		//			ABCD		ABCG		ABCD		LBJD
		//			EFGH	=>	EFJH		EFGH	=>	EFGH
		//			IJKL		IMKL		IJKL		IAKC
		//			MNOP		DNOP		MNOP		MNOP
		grid2d_Shift(OriginX=1,OriginY=1,AmountX=0,AmountY=0)
			if ( !isnum(OriginX) || !isnum(OriginY) )	return FALSE
			AmountX										= src.grid2d_GetWrapX(AmountX)
			AmountY										= src.grid2d_GetWrapY(AmountY)
			if (AmountX==src.grid2d_GetMaxX())			AmountX	= 0
			if (AmountY==src.grid2d_GetMaxY())			AmountY	= 0
			if (!AmountX && !AmountY)					return TRUE
			OriginX	= src.grid2d_GetWrapX(OriginX)
			OriginY	= src.grid2d_GetWrapY(OriginY)
			var
				//	Save a value.
				value	= src.grid2d_GetItem(OriginX,OriginY)
				//	Find what should be listed in that value's former spot.
				X		= src.grid2d_GetWrapX(OriginX-AmountX)
				Y		= src.grid2d_GetWrapY(OriginY-AmountY)
			src.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),OriginX,OriginY)
			//	Find out what goes in the slot left by the prior item.
			//	Move on to the spot left by that.  And so on.
			var
				nextX
				nextY
			//	Quit when back to the original spot.
			while((X!=OriginX) || (Y!=OriginY))
				nextX	= src.grid2d_GetWrapX(X-AmountX)
				nextY	= src.grid2d_GetWrapY(Y-AmountY)
				src.grid2d_PlaceItem(src.grid2d_GetItem(nextX,nextY),X,Y)
				X		= nextX
				Y		= nextY
			//	Place the saved value.
			src.grid2d_PlaceItem(value,src.grid2d_GetWrapX(OriginX+AmountX),src.grid2d_GetWrapY(OriginY+AmountY))
			return TRUE

		//----------------------------------------------------------------------
		//	Moves the item at OriginX,OriginY the specified amount in direction Dir.
		//	Items in cells along the line formed will be similarly moved.
		//	Unlike, grid2d_Shift(), this does not skip the spaces between.
		grid2d_ShiftDir(OriginX=1,OriginY=1,Dir=0,Amount=1)
			if ( !isnum(OriginX) || !isnum(OriginY) )				return FALSE
			if (!isnum(Dir) || !isnum(Amount) || !Dir || !Amount)	return FALSE
			OriginX	= src.grid2d_GetWrapX(OriginX)
			OriginY	= src.grid2d_GetWrapY(OriginY)
			//	Find the offsets.
			var
				offsetX					= 0
				offsetY					= 0
			if (Dir&EAST)
				if (Amount<0)			offsetX	= -1
				else					offsetX	= 1
			else if (Dir&WEST)
				if (Amount<0)			offsetX	= 1
				else					offsetX	= -1
			if (Dir&NORTH)
				if (Amount<0)			offsetY	= -1
				else					offsetY	= 1
			else if (Dir&SOUTH)
				if (Amount<0)			offsetY	= 1
				else					offsetY	= -1
			if (!offsetX && !offsetY)	return FALSE
			Amount	= abs(Amount)
			//	Move items.
			var
				value
				X
				Y
				nextX
				nextY
			while(Amount>0)
				Amount--
				//	Save a value.
				value	= src.grid2d_GetItem(OriginX,OriginY)
				//	Find what should be listed in that value's former spot.
				X		= src.grid2d_GetWrapX(OriginX-offsetX)
				Y		= src.grid2d_GetWrapY(OriginY-offsetY)
				src.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),OriginX,OriginY)
				//	Find out what goes in the slot left by the prior item.
				//	Move on to the spot left by that.  And so on.
				//	Quit when back to the original spot.
				while((X!=OriginX) || (Y!=OriginY))
					nextX	= src.grid2d_GetWrapX(X-offsetX)
					nextY	= src.grid2d_GetWrapY(Y-offsetY)
					src.grid2d_PlaceItem(src.grid2d_GetItem(nextX,nextY),X,Y)
					X		= nextX
					Y		= nextY
				//	Place the saved value.
				src.grid2d_PlaceItem(value,src.grid2d_GetWrapX(OriginX+offsetX),src.grid2d_GetWrapY(OriginY+offsetY))
			return TRUE


		//----------------------------------------------------------------------
		//	Moves the items at the given coordinates to the Amount next coordinates
		//	along the list.  Items at opposite ends of the list wrap.  In other
		//	words this moves items along an arbitrary path.
		grid2d_ShiftPath(list/PathCoords,Amount=1)
			if ( !istype(PathCoords,/list) || !isnum(Amount) )	return FALSE
			if ((PathCoords.len<=2) || (PathCoords.len%2))	return FALSE
			//	Get good coordinates.  Need wrap values anyway.
			var
				index
				list/goodCoords	= list()
			for(index = 1 to PathCoords.len/2)
				//	One bad apple spoils the whole damn bunch!
				if ( !isnum(PathCoords[2*index-1]) || !isnum(PathCoords[2*index]) )	return FALSE
				goodCoords	+= src.grid2d_GetWrapX(PathCoords[2*index-1])
				goodCoords	+= src.grid2d_GetWrapY(PathCoords[2*index])
			//	Check amount after coords so TRUE doesn't give false hope.
			Amount	= src.grid2d_GetWrapMin1(Amount,goodCoords.len/2)
			if (!Amount || (Amount==goodCoords.len/2))	return TRUE
			//	Find the items.
			var/list/items	= list()
			for(index = 1 to goodCoords.len/2)
				items	+= src.grid2d_GetItem(goodCoords[2*index-1],goodCoords[2*index])
			//	Place the items.
			for(index = 1 to items.len)
				src.grid2d_PlaceItem( \
					items[src.grid2d_GetWrapMin1(index-Amount,items.len)],
					goodCoords[2*index-1], \
					goodCoords[2*index] \
					)
			return TRUE
