grid2d_Grid

	//==========================================================================
	proc
		/*
			Turning performs the following transformations based on the clockwise
			angle.  That angle is expected to be a multiple of 90 degrees.

				origin		90 or -270		180 or -180		270 or -90
				ABC				GDA				IHG				CFI
				DEF				HEB				FED				BEH
				GHI				IFC				CBA				ADG

			If the dimensions of the grid are not equal, they might be swapped
			in the result.

				origin		90 or -270		180 or -180		270 or -90
								DA								CF
				ABC				EB				FED				BE
				DEF				FC				CBA				AD
		*/

		//----------------------------------------------------------------------
		//	Get the horizontal coordinate of the X,Y cell as if the grid was turned
		//	Angle degrees.  Returns 0 on failure.
		grid2d_GetTurnX(X,Y,Angle)
			if ( !isnum(Angle) || (Angle%90) )		return 0
			if ( !src.grid2d_HasCoordinates(X,Y) )	return 0
			if ( src.grid2d_DoesWrapX() )			X	= src.grid2d_GetWrapX(X)
			if ( src.grid2d_DoesWrapY() )			Y	= src.grid2d_GetWrapY(Y)
			Angle									= Angle%360
			if ( (Angle==90) || (Angle==-270) )		return Y
			if (abs(Angle)==180)					return src.grid2d_GetMaxX() - X + 1
			if ( (Angle==270) || (Angle==-90) )		return src.grid2d_GetMaxY() - Y + 1
			return X

		//----------------------------------------------------------------------
		//	Get the vertical coordinate of the X,Y cell as if the grid was turned
		//	Angle degrees.  Returns 0 on failure.
		grid2d_GetTurnY(X,Y,Angle)
			if ( !isnum(Angle) || (Angle%90) )		return 0
			if ( !src.grid2d_HasCoordinates(X,Y) )	return 0
			if ( src.grid2d_DoesWrapX() )			X	= src.grid2d_GetWrapX(X)
			if ( src.grid2d_DoesWrapY() )			Y	= src.grid2d_GetWrapY(Y)
			Angle									= Angle%360
			if ( (Angle==90) || (Angle==-270) )		return src.grid2d_GetMaxX() - X + 1
			if (abs(Angle)==180)					return src.grid2d_GetMaxY() - Y + 1
			if ( (Angle==270) || (Angle==-90) )		return X
			return Y

		//----------------------------------------------------------------------
		//	Returns a grid turned clockwise by Angle degrees.
		grid2d_GetTurnGrid(Angle)
			if ( !isnum(Angle) || (Angle%90) )	return null
			Angle								= Angle%360
			if (!Angle)							return src.grid2d_Copy()
			var
				maxX							= src.grid2d_GetMaxX()
				maxY							= src.grid2d_GetMaxY()
				grid2d_Grid/result
				X
				Y
			if ( (Angle==90) || (Angle==-270) )
				result	= new src.type (maxY,maxX)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),Y,src.grid2d_GetMaxX() - X + 1)
				return result
			if ( (Angle==180) || (Angle==-180) )
				result	= new src.type (maxX,maxY)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),src.grid2d_GetMaxX() - X + 1,src.grid2d_GetMaxY() - Y + 1)
				return result
			if ( (Angle==270) || (Angle==-90) )
				result	= new src.type (maxY,maxX)
				for(X = 1 to maxX)
					for(Y = 1 to maxY)
						result.grid2d_PlaceItem(src.grid2d_GetItem(X,Y),src.grid2d_GetMaxY() - Y + 1,X)
				return result
			return null
