/*
	Wrapping brings coordinates in bounds as if they exited one side just
	to enter the opposite side.
*/

grid2d_Grid

	//==========================================================================
	var
		const
			grid2d_WRAP_NONE	= 0
			grid2d_WRAP_X		= 1
			grid2d_WRAP_Y		= 2
		grid2d_wrap	= grid2d_WRAP_NONE

	//==========================================================================
	proc
		//----------------------------------------------------------------------
		//	Returns whether or not cells on the left wrap to the right and vice versa.
		grid2d_DoesWrapX()
			return src.grid2d_wrap&src.grid2d_WRAP_X

		//----------------------------------------------------------------------
		//	Returns whether or not cells on the bottom wrap to the top and vice versa.
		grid2d_DoesWrapY()
			return src.grid2d_wrap&src.grid2d_WRAP_Y

		//----------------------------------------------------------------------
		//	Returns 0 on failure.  Integer between 1 and Max inclusive otherwise.
		grid2d_GetWrapMin1(Val,Max)
			if ( !isnum(Val) || !isnum(Max) )	return 0
			Val									= round(Val)
			return ( (Val<0) ? ((Val%Max)+Max) : ((Val%Max) || Max) )

		//----------------------------------------------------------------------
		//	Find the value of X as if it went through the WEST or EAST sides.
		//	Returns 0 on failure.
		grid2d_GetWrapX(X)
			return src.grid2d_GetWrapMin1(X,src.grid2d_X)

		//----------------------------------------------------------------------
		//	Find the value of Y as if it went through the SOUTH or NORTH sides.
		//	Returns 0 on failure.
		grid2d_GetWrapY(Y)
			return src.grid2d_GetWrapMin1(Y,src.grid2d_cells.len/src.grid2d_X)

		//----------------------------------------------------------------------
		//	Sets whether or not cells on the edges wrap across the horizontal
		//	and/or vertical axes.
		grid2d_SetWrap(X=FALSE,Y=FALSE)
			if (X)	src.grid2d_wrap	|= src.grid2d_WRAP_X
			else	src.grid2d_wrap	&= ~src.grid2d_WRAP_X
			if (Y)	src.grid2d_wrap	|= src.grid2d_WRAP_Y
			else	src.grid2d_wrap	&= ~src.grid2d_WRAP_Y
			return TRUE
